<?php
/*********************************************************************************
 * Copyright (c) 1999 - 2008 Omni Technology Solutions, Inc.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 ********************************************************************************
 *
 * This module is an extension for SugarCRM developed by Omni Technology Solutions
 * to allow SugarCRM Community Edition to support user impersonation.
 *
 * Riva is an application integration framework developed by
 * Omni Technology Solutions, Inc.  Copyright (C) 1999 - 2008
 *
 * SugarCRM is a customer relationship management program developed by
 * SugarCRM, Inc. Copyright (C) 2004 - 2007 SugarCRM Inc.
 *
 * You can contact Omni Technology Solutions, Inc. headquarters at #1200 Bell Tower,
 * 10104-103 Ave, Edmonton, AB T5J 0H8, Canada. or via email support@omni-ts.com.
 *
 ********************************************************************************/

require_once('soap/SoapError.php');
require_once('soap/SoapHelperFunctions.php');
require_once('soap/SoapTypes.php');
require_once('modules/Users/User.php');

/**
 *
 * Sudo is the process of impersonation used to act on behalf of a user without knowledge of their password.
 *  
 **/
$server->register(
    'sudo_user',
    array('session'=>'xsd:string', 'user_name'=>'xsd:string'),
    array('return'=>'tns:error_value'),
    $NAMESPACE);
 
/**
 *
 * This function requires a session be created first and that it be admin-equivalent.
 *
 * @param String $session -- The session id that was created during the login() call.
 * @param String $user_name -- Name of the user to impersonate.
 * @return invalid_login
 * @return no_access 
 */
function sudo_user($session, $user_name)
{
	$error = new SoapError();

	/// Input validation
	if(empty($session) || empty($user_name))
	{
		$error->set_error('invalid_login');
		return $error->get_soap_array();
	}

	/// Ensure the account has been authenticated.
	if(!validate_authenticated($session))
	{
		$error->set_error('invalid_login');
		return $error->get_soap_array();
	}

	global $current_user;

	/// Only Admin users can use the functionality.
	/// Note: Here we could potentially add an additional ACL
	if(!is_admin($current_user))
	{
		/// oopps.... Not admin-equivalent!
		$error->set_error('no_access');
	}
	else
	{		
		/// Initialization
		$sudo_user = new User();

		/// Get guid for $user_name
		$sudo_id = $sudo_user->retrieve_user_id($user_name);

		if(!$sudo_id)
		{
			/// $user_name does not exist.
			$error->set_error('invalid_login');
		}
		else
		{
			/// $user_name exists.
		
			/// Fill the $sudo_user object
			$sudo_user->retrieve($sudo_id);

			/// Do "Impersonate"
			$current_user = $sudo_user;
			$_SESSION['user_id'] = $sudo_id;

			/// Note: During the login, the following session variables are set.  So, at this point to properly
			/// complete the login we may want to set them for new current_user?
			//$current_user->loadPreferences();
			//$_SESSION['is_valid_session']= true;
			//$_SESSION['ip_address'] = $_SERVER['REMOTE_ADDR'];
			//$_SESSION['user_id'] = $current_user->id;
			//$_SESSION['type'] = 'user';
			//$_SESSION['avail_modules']= get_user_module_list($current_user);
			//$_SESSION['authenticated_user_id'] = $current_user->id;
			//$_SESSION['unique_key'] = $sugar_config['unique_key'];
		}
	}

	/// Return results
	return $error->get_soap_array();
}
?>
